/***************************************************************************\
  "input.h" - input buffer
  Rev 1; 10 Oct 2002
  Copyright (C) 2002  Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef INPUT_H
#define INPUT_H

/////////////////////////
// DEFINES //////////////
/////////////////////////
#define INPUT_LINE 256
#define INPUT_EOF 127

/////////////////////////
// INCLUDES /////////////
/////////////////////////
#include "k_string.h"
#include "ohno.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////
void input_start (char *filename);
char input_get ();
bool input_skipwhite ();

char *input_text;
int input_pos;

void input_start (char *filename)
{
  FILE *input_file;
  char line[INPUT_LINE];

  input_file = fopen (filename, "r");
  input_pos = 0;
  if (!input_file)
  {
    ohno (true, "file not found", filename, NULL);
  }

  input_text = k_new (32768);
  k_copy (input_text, "");

  while (!feof (input_file))
  {
    if (fgets (line, INPUT_LINE, input_file) != NULL)
    {
      k_cat (input_text, line);
    }
  }

  fclose (input_file);
}

char input_get ()
{
  if (input_skipwhite ()) return ' ';

  if (input_pos < (signed)strlen (input_text))
  {
    return input_text[input_pos++];
  }
  else return INPUT_EOF;
}

bool input_skipwhite ()
{
  int len;
  bool skipped;

  len = strlen (input_text);
  skipped = false;

  while ( (!k_inSet (input_text[input_pos], K_LETTERS)) &&
          (input_pos < len)
        )
  {
    skipped = true;
    input_pos++;
  }

  return skipped;
}

#endif
