/***************************************************************************\
  "interpreter.h" - interpreter
  Rev 1; 10 Oct 2002
  Copyright (C) 2002  Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef INTERPRETER_H
#define INTERPRETER_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define INTERPRETER_INPUT_LINE 256

/////////////////////////
// INCLUDES /////////////
/////////////////////////

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

// Private //

// Public //
void interpreter_interpret ();

// Private - Interpret Instructions //
void interpret_SET ();
void interpret_PRINT (); 
void interpret_GETN ();
void interpret_GETS ();
void interpret_ADD ();
void interpret_SUB ();
void interpret_MUL ();
void interpret_DIV ();
void interpret_LABEL ();
void interpret_GOTO ();
void interpret_JLT ();
void interpret_JGT ();
void interpret_JE ();
void interpret_JNE ();

////////////////////////
// DEFINITIONS /////////
////////////////////////

void interpreter_interpret ()
{
  for (ip = 0; ((memory[ip].code != wcNull) && (ip < MEMORY_SIZE)); ip++)
  {
    switch (memory[ip].code)
    {
      case wcSET:
        interpret_SET ();
        break;
      case wcPRINT:
        interpret_PRINT (); 
        break;
      case wcGETN:
        interpret_GETN ();
        break;
      case wcGETS:
        interpret_GETS ();
        break;
      case wcADD:
        interpret_ADD ();
        break;
      case wcSUB:
        interpret_SUB ();
        break;
      case wcMUL:
        interpret_MUL ();
        break;
      case wcDIV:
        interpret_DIV ();
        break;
      case wcLABEL: 
        interpret_LABEL ();
        break;
      case wcGOTO:
        interpret_GOTO ();
        break;
      case wcJLT:
        interpret_JLT ();
        break;
      case wcJGT:
        interpret_JGT ();
        break;
      case wcJE:
        interpret_JE ();
        break;
      case wcJNE:
        interpret_JNE ();
        break;
      default:
        ohno (true, "RUN!!! ..RUn!! Inva1id 1npu7.=. dying. %DyIg ... dYINGGGG#GGGG@#G23G\nGG@GGG2GG!@#GGGGGGG*(@!GGGGGGGG\nGGG\nGGGGGGGGG", NULL, NULL);
    }
  }
}

void interpret_SET ()
{
  memory[ip].a->type = memory[ip].b->type;

  switch (memory[ip].a->type)
  {
    case tString:
      if (memory[ip].a->value == NULL) memory[ip].a->value = k_clone ("");
      k_copy (memory[ip].a->value, memory[ip].b->value);
      break;
    case tNumber:
      if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
      *((int*)(memory[ip].a->value)) = *((int*) memory[ip].b->value);
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_PRINT ()
{
  switch (memory[ip].a->type)
  {
    case tString:
      printf ("%s", (char*)(memory[ip].a->value));
      break;
    case tNumber:
      printf ("%d", *((int*)(memory[ip].a->value)));
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_GETN ()
{
  char line[INTERPRETER_INPUT_LINE];

  memory[ip].a->type = tNumber;
  gets (line);

  if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
  *((int*)(memory[ip].a->value)) = atoi (line);
}

void interpret_GETS ()
{
  char line[INTERPRETER_INPUT_LINE];

  memory[ip].a->type = tString;
  gets (line);

  if (memory[ip].a->value == NULL) memory[ip].a->value = k_clone ("");
  k_copy (memory[ip].a->value, line);
}

void interpret_ADD ()
{
  memory[ip].a->type = memory[ip].b->type;

  switch (memory[ip].a->type)
  {
    case tNumber:
      if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
      *((int*)(memory[ip].a->value)) = *((int*) memory[ip].a->value) + *((int*) memory[ip].b->value);
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_SUB ()
{
  memory[ip].a->type = memory[ip].b->type;

  switch (memory[ip].a->type)
  {
    case tNumber:
      if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
      *((int*)(memory[ip].a->value)) = *((int*) memory[ip].a->value) - *((int*) memory[ip].b->value);
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_MUL ()
{
  memory[ip].a->type = memory[ip].b->type;

  switch (memory[ip].a->type)
  {
    case tNumber:
      if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
      *((int*)(memory[ip].a->value)) = *((int*) memory[ip].a->value) * *((int*) memory[ip].b->value);
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_DIV ()
{
  memory[ip].a->type = memory[ip].b->type;

  switch (memory[ip].a->type)
  {
    case tNumber:
      if (memory[ip].a->value == NULL) memory[ip].a->value = alloc_n (0);
      *((int*)(memory[ip].a->value)) = *((int*) memory[ip].a->value) / *((int*) memory[ip].b->value);
      break;
    default:
      ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
}

void interpret_LABEL ()
{
  // empty //
}

void interpret_GOTO ()
{
  ohno (true, "goto not yet implemented", NULL, NULL);
}

void interpret_JLT ()
{
  if ((memory[ip].a->type != tNumber) || (memory[ip].a->type != tNumber))
  {
    ohno (true, "type mismatch, expected number (instruction:type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
  else
  {
    if (*((int*) memory[ip].a->value) < *((int*) memory[ip].b->value))
    {
      ip = *((int*)(memory[ip].c->value));
    }
  }
}  

void interpret_JGT ()
{
  if ((memory[ip].a->type != tNumber) || (memory[ip].a->type != tNumber))
  {
    ohno (true, "type mismatch, expected number (instruction:type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
  else
  {
    if (*((int*) memory[ip].a->value) > *((int*) memory[ip].b->value))
    {
      ip = *((int*)(memory[ip].c->value));
    }
  }
}

void interpret_JE ()
{
  if (memory[ip].a->type != memory[ip].b->type)
  {
    ohno (true, "type mismatch (instruction)", WordCodeStrings[memory[ip].code], NULL);
  }
  else
  {
    switch (memory[ip].a->type)
    {
      case tString:
        if (k_same (memory[ip].a->value, memory[ip].b->value))
        {
          ip = *((int*)(memory[ip].c->value));
        }
        break;
      case tNumber:
        if (*((int*) memory[ip].a->value) == *((int*) memory[ip].b->value))
        {
          ip = *((int*)(memory[ip].c->value));
        }
        break;
      default:
        ohno (true, "invalid type (instruction:variable type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
    }
  }
}

void interpret_JNE ()
{
  if ((memory[ip].a->type != tNumber) || (memory[ip].a->type != tNumber))
  {
    ohno (true, "type mismatch, expected number (instruction:type)", WordCodeStrings[memory[ip].code], TypeStrings[memory[ip].a->type]);
  }
  else
  {
    if (*((int*) memory[ip].a->value) != *((int*) memory[ip].b->value))
    {
      ip = *((int*)(memory[ip].c->value));
    }
  }
}

#endif
