/***************************************************************************\
  "lexer.h" - lexer
  Rev 1; 10 Oct 2002
  Copyright (C) 2002  Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef LEXER_H
#define LEXER_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define LEXER_WORD_LEN 80

/////////////////////////
// INCLUDES /////////////
/////////////////////////

#include "k_string.h"
#include "ohno.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

typedef enum
{
  wcNull,
  wcSET,
  wcADD, wcSUB, wcMUL, wcDIV,
  wcLABEL, wcGOTO,
  wcJLT, wcJGT, wcJE, wcJNE,
  wcPRINT, wcGETN, wcGETS,
  wcSTROPEN, wcSTRCLOSE, wcNUMBER
} WordCode;

char *WordCodeStrings[] = 
{
  "null",
  "set",
  "add", "sub", "mul", "div",
  "label", "goto",
  "jlt", "jgt", "je", "jne",
  "print", "getn", "gets",
  "stropen", "strclose", "number"
};

typedef struct
{
  int value;
  WordCode code;
  char *variable;
  char *string;
} Word;

void word_del (Word *word);
Word *lexer_next ();
int lexer_getvalue (char c);
WordCode lexer_getcode (int value, char *wordstring);

void word_del (Word *word)
{
  if (!word) return;

  if (word->variable) free (word->variable);
  if (word->string) free (word->string);
  free (word);
}

Word *lexer_next ()
{
  char c;
  int value;
  Word *word;
  char buf[LEXER_WORD_LEN];
  char pos;

  c = input_get ();
  value = 0;
  pos = 0;

  while ((c != ' ') && (c != INPUT_EOF))
  {
    value += lexer_getvalue (c);

    buf[pos++] = c;
    c = input_get ();
  }

  if (c == INPUT_EOF)
  {
    return NULL;
  }
  else
  {
    buf[pos] = '\0';

    word = (Word *) malloc (sizeof (Word));
    word->value = value;
    word->variable = k_clone ("var_");
    k_catd (word->variable, k_int (word->value));
    word->string = k_clone (buf);
    word->code = lexer_getcode (word->value, word->string);

    return word;
  }
}

int lexer_getvalue (char c)
{
  switch (tolower(c))
  {
    case 'a':
    case 'b':
    case 'c':
      return 2;
      break;
    case 'd':
    case 'e':
    case 'f':
      return 3;
      break;
    case 'g':
    case 'h':
    case 'i':
      return 4;
      break;
    case 'j':
    case 'k':
    case 'l':
      return 5;
      break;
    case 'm':
    case 'n':
    case 'o':
      return 6;
      break;
    case 'p':
    case 'q':
    case 'r':
    case 's':
      return 7;
      break;
    case 't':
    case 'u':
    case 'v':
      return 8;
      break;
    case 'w':
    case 'x':
    case 'y':
    case 'z':
      return 9;
      break;
    default:
      ohno (true, "i don't understand the input to the lexer (character:ASCII value)", c2string(c), k_int((int) c));
  }
}

WordCode lexer_getcode (int value, char *wordstring)
{
  if (value < 2) ohno (true, "i am really confused!!", NULL, NULL);
  else if ((value >=  2) && (value <= 10)) return wcSET;
  else if ((value >= 11) && (value <= 12)) return wcPRINT;
  else if ((value >= 13) && (value <= 14)) return wcGETN;
  else if ((value >= 15) && (value <= 16)) return wcGETS;
  else if ((value >= 17) && (value <= 18)) return wcSTROPEN;
  else if ((value >= 19) && (value <= 20)) return wcSTRCLOSE;
  else if ((value >= 21) && (value <= 22)) return wcNUMBER;
  else if ((value >= 23) && (value <= 24)) return wcADD;
  else if ((value >= 25) && (value <= 26)) return wcSUB;
  else if ((value >= 27) && (value <= 28)) return wcMUL;
  else if ((value >= 29) && (value <= 30)) return wcDIV;
  else if ((value >= 31) && (value <= 32)) return wcLABEL;
  else if ((value >= 33) && (value <= 34)) return wcGOTO;
  else if ((value >= 35) && (value <= 36)) return wcJLT;
  else if ((value >= 37) && (value <= 38)) return wcJGT;
  else if ((value >= 39) && (value <= 40)) return wcJE;
  else if ((value >= 41) && (value <= 42)) return wcJNE;
  else                                     return wcNull;
}    

#endif
