/***************************************************************************\
  "memory.h" - memory module holding the intermediate code
  Rev 1; 10 Oct 2002
  Copyright (C) 2002  Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

#ifndef MEMORY_H
#define MEMORY_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

#define MEMORY_SIZE 1024

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "lexer.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

typedef struct
{
  WordCode code;
  Symbol *a, *b, *c;
} Instruction;

Instruction *memory; 
int ip;

void memory_start ();
void memory_stop ();
void memory_reset ();
void memory_write (WordCode code, Symbol *a, Symbol *b, Symbol *c);

void memory_start ()
{
  int i;

  ip = 0;

  memory = (Instruction *) malloc (sizeof (Instruction) * MEMORY_SIZE);

  for (i = 0; i < MEMORY_SIZE; i++)
  {
    memory[i].code = wcNull;
  }
}

void memory_stop ()
{
  free (memory);
}

void memory_reset ()
{
  ip = 0;
}

void memory_write (WordCode code, Symbol *a, Symbol *b, Symbol *c)
{
  memory[ip].code = code;
  memory[ip].a = a;
  memory[ip].b = b;
  memory[ip].c = c;

  ip++;
}

#endif
