/***************************************************************************\
  "numpad.c" - numpad interpreter
  Copyright (C) 2002  Kevin P. Albrecht

  Permission is hereby granted by Kevin P. Albrecht (the "Author"), free of
  charge, to any person obtaining a copy of this software and associated
  documentation files (the "Software"), to deal in the Software without
  restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the
  Software, and to permit persons to whom the Software is furnished to do
  so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included
  in all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
  THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
  AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

  Except as contained in this notice, the name of the Author shall not be
  used in advertising or otherwise to promote the sale, use or other
  dealings in this Software without prior written authorization from the
  Author.
\***************************************************************************/

/////////////////////////
// DEFINES //////////////
/////////////////////////

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //
#include <stdio.h>
#include <stdlib.h>

// Global Declarations //
#include "table.h"
Table *gTable;

// Local //
#include "global.h"
#include "input.h"
#include "ohno.h"
#include "k_string.h"
#include "memory.h"
#include "parser.h"
#include "interpreter.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

int main (int nArgs, char *vsArgs[])
{
  // Vars - Declare //
  char *filenameInput;
  char *filenameOutput;
  bool interpret;       // true: interpret, false: output icode
  int inArg;
  FILE *fileOutput;
  int x, y;

  // Vars - Define //
  filenameInput = k_new (256); k_copy (filenameInput, "I AM EMPTY!!");
  filenameOutput = k_new (256); k_copy (filenameOutput, "I AM EMPTY!!");
  interpret = true;
  
  // Command-line Argument Handling //
  if (nArgs > 1)
  {
    for (inArg = 1; inArg < nArgs; inArg++)
    {
      // '-a', '--about' //
      if ((k_same (vsArgs[inArg], "-a")) || (k_same (vsArgs[inArg], "--about")))
      {
        puts ("");
        puts ("========================= NUMPAD Interpreter =========================");
        puts ("");
        puts ("Notice:  Copyright (C) 2002, Kevin P. Albrecht");
        puts ("E-mail:  kevin@albrecht.net");
        puts ("Website: http://kevin.alteu.com/");
        puts ("");
        puts ("Version: 1.0 (10 Oct 2002)");
        puts ("Build:   0050");
        return 0;
      }
      // '-l', '--license' //
      if ((k_same (vsArgs[inArg], "-l")) || (k_same (vsArgs[inArg], "--license")))
      {
        puts ("");
        puts ("Copyright (C) 2000-2002, Kevin P. Albrecht");
        puts ("");
        puts ("Permission is hereby granted by Kevin P. Albrecht (the \"Author\"), free of");
        puts ("charge, to any person obtaining a copy of this software and associated");
        puts ("documentation files (the \"Software\"), to deal in the Software without");
        puts ("restriction, including without limitation the rights to use, copy, modify,");
        puts ("merge, publish, distribute, sublicense, and/or sell copies of the");
        puts ("Software, and to permit persons to whom the Software is furnished to do");
        puts ("so, subject to the following conditions:");
        puts ("The above copyright notice and this permission notice shall be included");
        puts ("in all copies or substantial portions of the Software.");
        puts ("");
        puts ("THE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR");
        puts ("IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,");
        puts ("FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL");
        puts ("THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN");
        puts ("AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN");
        puts ("CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.");
        puts ("");
        puts ("Except as contained in this notice, the name of the Author shall not be");
        puts ("used in advertising or otherwise to promote the sale, use or other");
        puts ("dealings in this Software without prior written authorization from the");
        puts ("Author.");
        return 0;
      }
      // '-v', '--verbose' //
      else if ((k_same (vsArgs[inArg], "-v")) || (k_same (vsArgs[inArg], "--verbose")))
      {
      }
      // '-o', '--output' //
      else if ((k_same (vsArgs[inArg], "-o")) || (k_same (vsArgs[inArg], "--output")))
      {
        if ((inArg+1) < nArgs)
        {
          interpret = false;
          k_copy (filenameOutput, vsArgs[++inArg]);
        }
        else
        {
          ohno (true, "no argument given for output file", NULL, NULL);
        }
      }
      // '-c', '--calc' //
      else if ((k_same (vsArgs[inArg], "-c")) || (k_same (vsArgs[inArg], "--calc")))
      {
        if ((inArg+1) < nArgs)
        {
          inArg++;

          for (x = 0, y = 0; vsArgs[inArg][x] != '\0'; x++)
          {
            y += lexer_getvalue (vsArgs[inArg][x]);
          }

          printf ("value: %d\n", y);
          printf ("code:  %s\n", WordCodeStrings[lexer_getcode(y, vsArgs[inArg])]);

          return 0;
        }
        else
        {
          ohno (true, "no argument given to calculate", NULL, NULL);
        }
      }
      else
      {
        if (vsArgs[inArg][0] == '-')
        {
          ohno (true, "one of the arguments is weird", vsArgs[inArg], NULL);
        }
        else
        {
          k_copy (filenameInput, vsArgs[inArg]);
          break;
        }
      }
    }
  }
  else
  {
    puts ("Usage: numpad [ option(s) ] [ input file ]");
    puts ("");
    puts ("Option           | Description                                      ");
    puts ("-----------------+--------------------------------------------------");
    puts ("-a, --about      | display release and contact info                 ");
    puts ("-l, --license    | display distribution license                     ");
    puts ("-v, --verbose    | print verbose information                        ");
    puts ("-o X, --output X | output intermediate code to filename X           ");
    puts ("-c X, --calc X   | calculate the value for for the word X           ");
    return 0;
  }

  // Verify the capture of the filenames //
  if (k_same (filenameInput, "I AM EMPTY!!"))
  {
    ohno (true, "give me a file name", NULL, NULL);
  }

  // Initialize Modules //
  global_start ();
  gTable = tab_new ();
  memory_start ();
  input_start (filenameInput);

  parser_parse ();

  if (interpret)
  {
    interpreter_interpret ();
  }
  else
  {
    fileOutput = fopen (filenameOutput, "w");    

    for (ip = 0; ((memory[ip].code != wcNull) && (ip < MEMORY_SIZE)); ip++)
    {
      fprintf (fileOutput, "%s ", WordCodeStrings[memory[ip].code]);
      if (memory[ip].a)
        fprintf (fileOutput, "(\"%s\":%s) ", memory[ip].a->key, TypeStrings[memory[ip].a->type]);
      if (memory[ip].b)
        fprintf (fileOutput, "(\"%s\":%s) ", memory[ip].b->key, TypeStrings[memory[ip].b->type]);
      if (memory[ip].c)
        fprintf (fileOutput, "(\"%s\":%s) ", memory[ip].c->key, TypeStrings[memory[ip].c->type]);
      fprintf (fileOutput, "\n");
    }
  }

  // Vars - Free //
  k_del (filenameInput);
  k_del (filenameOutput);

  return 0;
}

